;________________________________________________________________________
;
;                            T I M I N G . A S M
;________________________________________________________________________
;
; Timing management code for APRS beacon - By IK3SVW Max - July 2002
; Based from code by Colin Buckup N5GG and Byon Garrabrant N6BG
;________________________________________________________________________

	cblock
		lastTMR0
		sub_second_timer
		quiet_timer
 		transmission_timer
		ptt_in_history
	endc
;________________________________________________________________________

Timing_Init
	movlw	D'1'
	movwf	beacon_counter
	movwf	status_counter
	movwf	sub_second_timer
	clrf	quiet_timer
	clrf	transmission_timer
	clrf	ptt_in_history
	return


Timing_Check_Update
	btfsc	VOX_IN			; clear quiet_timer
	 clrf	quiet_timer		;  if voice activity
	movfw	lastTMR0		; get difference from
	subwf	TMR0, W			; last time and
	addwf	lastTMR0, F		; add to last time
	skpc				; if no wrap
	 return				;  leave

	clrc				; prepare to update ptt history
	btfss	PTT_IN			; if PTT_IN pressed
	 setc				;  store status in ptt history
	rlf	ptt_in_history, F	; update ptt history

	decfsz	sub_second_timer, F	; count down the sub seconds
	 return				; if not zero, leave

	movlw	D'38'			; reset the sub seconds
	movwf	sub_second_timer	

	incf	quiet_timer, F		; bump quiet_timer
	skpnz				; but clamp
	 decf	quiet_timer, F		; at 255

	btfsc	VOX_IN			; clear quiet_timer
	 clrf	quiet_timer		;  if voice activity

	incf	transmission_timer, F	; inc transmission_timer
	skpnz				; but clamp
	 decf	transmission_timer, F	; to 255

	return
	

Timing_Check
	call	Timing_Check_Update

	btfsc	ptt_in_history, 0	; if PTT_IN pressed
	 bsf	PTT_OUT			;  set PTT_OUT on

	movfw	ptt_in_history		; check ptt history, and
	xorlw	H'FE'			; if PTT_IN just released
	bz	Do_Transmit_Manual	; transmit manual beacon

	tstf	ptt_in_history		; if no PTT_IN recently
	bnz	Check_Done

	movlw	OPT_RANGE_TIME		; check range time base
	call	Get_Option
	subwf	transmission_timer, W
	bnc	Check_Done

	movlw	OPT_QUIET_TIME		; check quiet time
	call	Get_Option
	subwf	quiet_timer, W
	bnc	Check_Done

	clrf	transmission_timer
	retlw	1

Do_Transmit_Manual
	movlw	OPT_MANUAL_TX_DELAY	; send initial flags
	call	AX25_Send_Start_Manual
	call	SendBeaconManual	; send the beacon packet
	clrc
	rlf	ptt_in_history, F
	clrf	transmission_timer

Check_Done	
	bcf	PTT_OUT			; release PTT
	retlw	0

;________________________________________________________________________